/*   procedure:     AddressEffectiveDateFunction.sql
     written by:    Craig Deane
     date written:  12/3/2020
     database:      welcoll
     function:      This function generates the Address Effective Date
     modified by:   
     changes: 	     	
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'AddressEffectiveDateFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.AddressEffectiveDateFunction
  END
go

create function dbo.AddressEffectiveDateFunction(@debtor_id integer)
returns datetime
begin
  declare @effectivedate datetime

  select @effectivedate = max(date_changed) from debtor_old_addresses where debtor_id = @debtor_id

  if @effectivedate is null
    select @effectivedate = min(date_entered) from debt where debtor_id = @debtor_id

  return @effectivedate

end
GO

grant execute on AddressEffectiveDateFunction to public
go


/*   procedure:     AddressFunction.sql
     written by:    Craig Deane
     date written:  9/15/2020
     database:      welcoll
     function:      This function returns a table with the Responsible Party address
     modified by:   Craig Deane 10/12/2020
     changes: 	    Modified for new file layout
     modified by:   Craig Deane 3/2020
     changes: 	    Modified the test to only match the test string if the next character is a number
     modified by:   Craig Deane 7/28/2021 FSD 1.8
     changes: 	    Modified the test to only match the test string if the next character is a space
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'AddressFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.AddressFunction
  END
go

create function dbo.AddressFunction(@street_address varchar(50))
returns @address table
(
	address_line1		varchar(50),
	address_line2		varchar(50)
)
as
begin
  declare @index 		integer,
	 @unit_position		integer,
	 @max_index		integer,
	 @test_unit		varchar(50)

 insert @address
 select @street_address,
	null

  select @max_index = max(key_number) from Line2Address
 
  select @index = 1

  while @index <= @max_index
  begin
   
    select @test_unit = unit from Line2Address where key_number = @index

    if IsNull(@test_unit, '') <> ''
    begin
      select @unit_position = CHARINDEX(' ' + @test_unit + ' ', @street_address, 1)
      
      if @unit_position > 0 
      begin
        update @address
	set address_line2 = substring(address_line1, @unit_position + 1, len(address_line1)), 
            address_line1 = substring(address_line1, 1, @unit_position)
    
	break
      end
    end

    select @index = @index + 1 

  end

  return
end
GO

grant select on AddressFunction to public
go


	/*   procedure:     AliasNameFunction.sql
     written by:    Craig Deane
     date written:  9/15/2020
     database:      welcoll
     function:      This function returns a table with the Responsible Party alias name
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'AliasNameFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.AliasNameFunction
  END
go

create function dbo.AliasNameFunction(@debtor_id integer)
returns @name table
(
	first_name	varchar(30),
	last_name	varchar(30),
	suffix_name	char(5)
)
as

begin
  declare @max integer

  select @max = max(alias_sequence_number)
  from alias_names 
  where debtor_id = @debtor_id

  insert @name
  select first_name,
	 (select last_name from dbo.NameSuffixFunction(alias_names.last_name)),
	 (select suffix_name from dbo.NameSuffixFunction(alias_names.last_name))

  from alias_names
  where debtor_id = @debtor_id and
        alias_sequence_number = @max

  return
end
GO

grant select on AliasNameFunction to public
go

/*   procedure:     CleanMemoFunction.sql
     written by:    Craig Deane
     date written:  3/3/2021
     database:      welcoll
     function:      This function replaces special characters in a string
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'CleanMemoFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.CleanMemoFunction
  END
go

create function dbo.CleanMemoFunction(@string varchar(max))
returns varchar(max)
as

begin
select @string = REPLACE(@string, CHAR(9), ' ')
select @string = REPLACE(@string, CHAR(10), '\n')
select @string = REPLACE(@string, CHAR(13), ' ')
select @string = REPLACE(@string, '"', '""')

return @string

end

GO

grant execute on CleanMemoFunction to public
go

/*   procedure:     DebtorNameFunction.sql
     written by:    Craig Deane
     date written:  10/19/2020
     database:      welcoll
     function:      This function returns a table with the Responsible Party name
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'DebtorNameFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.DebtorNameFunction
  END
go

create function dbo.DebtorNameFunction(@debtor_id integer, @debtor_number integer)
returns @name table
(
	first_name	varchar(25),
	middle_name	char(1),
	last_name	varchar(25),
	suffix_name		char(5)
)
as

begin

 declare @first_name	varchar(15),
	 @middle_name	char(1),
	 @last_name	varchar(20),
	 @suffix_name	varchar(5)

  if @debtor_number = 1
  begin
    select @first_name = debtor1_name_first,
	 @middle_name = debtor1_name_mid_init,
         @last_name = debtor1_name_last
    from debtor
    where debtor_id  = @debtor_id
  end
  else if @debtor_number = 2
  begin
    select @first_name = debtor2_name_first,
	@middle_name = debtor2_name_mid_init,
        @last_name = debtor2_name_last
    from debtor
    where debtor_id  = @debtor_id
  end

  select @suffix_name = suffix_name from dbo.NameSuffixFunction(@last_name)
  select @last_name = last_name from dbo.NameSuffixFunction(@last_name)

  insert @name
  select @first_name,
	 @middle_name,
	 @last_name,
	 @suffix_name

  return
end
GO

grant select on DebtorNameFunction to public
go

/*   	procedure:     	LegacyAccountNumberIdentifierFunction.sql
     	written by:    	Craig Deane
     	date written:  	12/11/2020
     	database:      	welcoll
     	function:      	This function generates the Legacy Account Number Identifier
     	modified by:   	Craig Deane 3/17/2021
     	changes: 	Converted it to an inline table valued function to improve performance
     	modified by:   	Craig Deane 7/28/2021 FSD 1.8
     	changes: 	Removed leading zeroes from the claim_number	
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'LegacyAccountNumberIdentifierFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.LegacyAccountNumberIdentifierFunction
  END
go

create function dbo.LegacyAccountNumberIdentifierFunction()
returns Table
as 
return  (select debt.debt_number,
		"LegacyAccountNumberIdentifier" = Case When IsNull(claim_number, '') = '' Then 'X' + CONVERT(varchar(10), debt.debt_number) 
		Else
			Case When cw_claim_number.loan_ind = 'Y' Then 'L' Else '' End + REPLACE(LTRIM(REPLACE(cw_claim_number.claim_number, '0', ' ')), ' ' , '0')
		End
	from debt
	LEFT JOIN cw_claim_number ON debt.debt_number = cw_claim_number.debt_number)
GO

grant select on LegacyAccountNumberIdentifierFunction to public
go

/*   
	procedure:     	NameSuffixFunction.sql
     	written by:    	Craig Deane
     	date written:  	10/19/2020
     	database:      	welcoll
     	function:      	This function returns a table with the last name and suffix
     	modified by:   	Craig Deane 5/20/2021
     	changes:	Fixed a bug where non-suffixes were being split off
     	modified by:   	Craig Deane 6/10/2021 FSD 1.7
     	changes:	Made it so that the next space after the test suffix must be blank
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'NameSuffixFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.NameSuffixFunction
  END
go

create function dbo.NameSuffixFunction(@last_name varchar(20))
returns @name table
(
	last_name	varchar(25),
	suffix_name	char(5)
)
as

begin

 declare @suffix_name	varchar(5),
	 @test_suffix	varchar(5),
	 @max_index	integer,
	 @index		integer,
	 @suffix_position	integer

  
  select @max_index = max(key_number) from NameSuffix
 
  select @index = 1

  while @index <= @max_index
  begin
   
    select @test_suffix = suffix from NameSuffix where key_number = @index

    if IsNull(@test_suffix, '') <> ''
    begin
      select @suffix_position = CHARINDEX(' ' + @test_suffix, @last_name, 1)
      
      if @suffix_position > 0 and IsNull(substring(@last_name, @suffix_position + len(@test_suffix) + 1, 1), '') = ''
      begin
        select @suffix_name = substring(@last_name, @suffix_position + 1, len(@test_suffix))
        select @last_name = substring(@last_name, 1, @suffix_position)
   
	break
      end
    end

    select @index = @index + 1 

  end

  insert @name
  select @last_name,
	 @suffix_name

  return
end
GO

grant select on NameSuffixFunction to public
go

/*   procedure:     OrganizationNameFunction.sql
     written by:    Craig Deane
     date written:  9/14/2020
     database:      welcoll
     function:      This function retrieves the Organization name for Foster Care debts
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'OrganizationNameFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.OrganizationNameFunction
  END
go

create function dbo.OrganizationNameFunction(@debt_number integer)
returns varchar(20)
begin
  declare @name varchar(20)

  select @name = debtor1_name_last
  from debt, debtor
  where debt.debtor_id = debtor.debtor_id and
	debt_number = @debt_number and
	type_code = 'FOS'
 
  return @name

end
GO

grant execute on OrganizationNameFunction to public
go
/*   procedure:     ResponsiblePartyBillIndicatorFunction.sql
     written by:    Craig Deane
     date written:  10/15/2020
     database:      welcoll
     function:      This function generates the Responsible Party Bill Indicator
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'ResponsiblePartyBillIndicatorFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.ResponsiblePartyBillIndicatorFunction
  END
go

create function dbo.ResponsiblePartyBillIndicatorFunction(@debt_number integer, @debtor_number integer)
returns char(2)
begin
  declare @debtor_id		integer,
	  @bill_indicator	char(1),
	  @debt_statement_ind	char(1),
	  @debtor_statement_ind	char(1)
	  

  select @debt_statement_ind = statement_ind
	from debt
	where debt_number = @debt_number

  select @debtor_statement_ind = Case When @debtor_number = 1 Then debtor1_statement_ind When @debtor_number = 2 Then debtor2_statement_ind End
	from debt, debtor
	where debt.debtor_id = debtor.debtor_id and
		debt_number = @debt_number


  if @debt_statement_ind = 'Y' and @debtor_statement_ind = 'Y' 
	select @bill_indicator = 'Y'
  Else
	select @bill_indicator = 'N'


   return @bill_indicator


end
GO

grant execute on ResponsiblePartyBillIndicatorFunction to public
go

/*   procedure:     RetrieveMemoFunction.sql
     written by:    Craig Deane
     date written:  10/13/2020
     database:      welcoll
     function:      This function retieves and formats the Activity memo
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'RetrieveMemoFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.RetrieveMemoFunction
  END
go

create function dbo.RetrieveMemoFunction(@activity_number integer)
returns varchar(2000)
begin
  declare @full_memo	varchar(2000),
	  @memo		varchar(max),	
	  @memo_text	varchar(max)

  select @memo = memo,
	 @memo_text = IsNull(convert(varchar(2000), memo_text), '')
  from user_activity
  where activity_number = @activity_number

  select @memo = REPLACE(@memo, '"', '""')
  select @memo_text = REPLACE(@memo_text, '"', '""')

  select @full_memo = substring(@memo + @memo_text, 1, 2000)

  return @full_memo
end
GO

grant execute on RetrieveMemoFunction to public
go

/*   procedure:     StatusCodeFunction.sql
     written by:    Craig Deane
     date written:  10/21/2020
     database:      welcoll
     function:      This function generates the Status Code
     modified by:   
     changes: 	     	
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'StatusCodeFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.StatusCodeFunction
  END
go

create function dbo.StatusCodeFunction(@debt_number integer)
returns char(2)
begin
  declare @status_code char(2)

  select @status_code = 
		(Case When debt_status = 'B' Then 'SU'
			When debt_status = 'C' Then 'CL'
			When debt_status = 'D' Then 'TE'
			When debt_status = 'H' Then 'SU'
			When debt_status = 'L' Then  'TE'
			When debt_status = 'O' Then  'AC'
			When debt_status = 'P' Then  'PE'
			When debt_status = 'S' Then 'SU'
			When debt_status = 'V' Then 'VO'
			When debt_status = 'W' Then 'TE'
			When debt_status = 'A' Then 'VO'
			Else null
		End)
  from debt 
  where debt_number = @debt_number

  return @status_code

end
GO

grant execute on StatusCodeFunction to public
go

/*   procedure:     StatusReasonCodeFunction.sql
     written by:    Craig Deane
     date written:  9/17/2020
     database:      welcoll
     function:      This function generates the Status Reason Code
     modified by:   Craig Deane 10/21/2020
     changes: 	    Modified for ne file specifications 	
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'StatusReasonCodeFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.StatusReasonCodeFunction
  END
go

create function dbo.StatusReasonCodeFunction(@debt_number integer)
returns char(2)
begin
  declare @status_reason_code char(2)

  select @status_reason_code = 
		(Case When debt_status = 'B' Then 'BR'
			When debt_status = 'C' Then 'PF'
			When debt_status = 'D' Then 'DE'
			When debt_status = 'H' Then 'AH'
			When debt_status = 'L' Then  'CL'
			When debt_status = 'O' and (debtor1_intercept_submitted= 'Y' or debtor2_intercept_submitted = 'Y') Then  'TI'
			When debt_status = 'O' and (IsNull(debtor1_intercept_submitted, 'N') = 'N' and IsNull(debtor2_intercept_submitted, 'N') = 'N') Then  'AC'
			When debt_status = 'P' Then  'IP'
			When debt_status = 'S' Then 'AD'
			When debt_status = 'V' Then 'AD'
			When debt_status = 'W' Then 'WO'
			Else null
		End)
  from debt 
  where debt_number = @debt_number

  return @status_reason_code

end
GO

grant execute on StatusReasonCodeFunction to public
go
/*   procedure:     VACSCaseStatusFunction.sql
     written by:    Craig Deane
     date written:  10/15/2020
     database:      welcoll
     function:      This returns the VACS Case Status or Program Status depending on how the Program Status Indicator is set
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'VACSCaseStatusFunction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP Function dbo.VACSCaseStatusFunction
  END
go

create function dbo.VACSCaseStatusFunction(@debt_number integer)
returns char(1)
begin
  declare @status_code	 	char(1),
	  @program_status_ind 	char(1),
	  @case_status		char(1),
	  @program_status	char(1)


  select @program_status_ind = program_status_ind
  from tax_intercept_control
  where tax_intercept_key = 'I'
  
 
  select @case_status = case_status
	from case_info, debt
	where case_info.reference_number = debt.reference_number and
	 	debt_number = @debt_number

  select @program_status = program_status
	from debt
	where debt_number = @debt_number


  if @program_status_ind = 'Y' 
	select @status_code = Case When IsNull(@program_status, @case_status) = 'A' Then 'Y' Else 'N' End

  if @program_status_ind = 'N'
	select @status_code = Case When IsNull(@case_status, @program_status) = 'A' Then 'Y' Else 'N' End

  return @status_code

end
GO

grant execute on VACSCaseStatusFunction to public
go
