/*   
	procedure:     	FillRecoveryAccount.prc
     	written by:    	Craig Deane
     	date written:  	July 2020
     	database:      	welcoll
     	function:      	This fills the RecoveryAccount table with claim data
     	modified by:   	Craig Deane 10/6/2020
     	changes:	Updated to reflect new layout
     	modified by:   	Craig Deane 10/7/2020
     	changes: 	Added Loan processing
     	modified by:   	Craig Deane 12/11/2020
     	changes: 	Added the LegacyAccountNumberIdentifierFunction that will return the debt_number when a debt does not have a claim_number
     	modified by:   	Craig Deane 2/2021
     	changes: 	Modified to use the DebtList table to filter results	
     	modified by:   	Craig Deane 3/10/2021 FSD 1.6
     	changes:       	Added the "CDS Case Indicator" field	
     	modified by:   	Craig Deane 3/17/2021
     	changes:       	Modified to use the new LegacyAccountNumberIdentifierFunction table valued function
     	modified by:   	Craig Deane 3/25/2021 FSD 1.7
     	changes:       	Renamed CDSCaseIndicator to NonCalWINCaseIndicator
     	modified by:   	Craig Deane 4/28/2021 FSD 1.7
     	changes: 	Modified the Organization Name processing. Fixed the Reason Code processing
     	modified by:   	Craig Deane 6/10/2021 FSD 1.7
     	changes: 	Added the county code as a default for the Staff ID	
    	modified by:   	Craig Deane 7/28/2021 FSD 1.8
     	changes: 	Modified the Staff ID processing to use the value from the VACSWorkers table when using the 			Vehicle option
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccount'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccount
  END
go

create procedure dbo.FillRecoveryAccount
as

declare 	@serial_type_ind	char(1),
		@staffIDIND 		integer,
		@county_code		char(2)

select @serial_type_ind = serial_type
  from calwin_control
 where calwin_key = 'C'

select @staffIDIND = staff_id_ind,
	@county_code = county_code
  from CalSAWSSettings
 where key_value = 1

delete RecoveryAccount 

insert RecoveryAccount 
select  "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,

	"Case Number" = (select case when @serial_type_ind = 'V' then substring(debt.reference_number, 5, 7) else
					substring(debt.reference_number, 3, 7) 	end),
									
	"Organization Name" = Case When IsNull(provider_number, '') <> '' or IsNull(debtor1_name_mid_init, '') = '*' Then debtor1_name_last End,
										
	"Organization ID" = provider_number,
									
	"Priority Number" = (select priority_number
				from PriorityNumber
				Where debt_number = debt.debt_number),
								
	"Program Code" = (select calsaws_code		
		from CalSAWSProgramCodeMap
		where CalSAWSProgramCodeMap.vacs_code = debt.type_code),
			
	"Type Code" = 'RE',

	"Expire Date" = lomeli_end_date,

	"Discovery Date" = date_of_discovery,

	"Staff ID" = 	Case 	When @staffIDIND = 1 Then 
					Case 	When IsNull((select worker_id
							from caseworker
							where case_info.worker_number = caseworker.worker_number), '') = '' Then @county_code
						else
							(select worker_id
							from caseworker
							where case_info.worker_number = caseworker.worker_number) 
					End
				When @staffIDIND = 2 Then 
					Case 	When IsNull((select CalWINUserID
							from VACSWorkers, debtor_financial_information
							where debtor_id = debt.debtor_id and
							debtor_number = 1 and 
							vehicle_year_make_model = VACSWorker), '') = '' Then @county_code
						else
							(select CalWINUserID
							from VACSWorkers, debtor_financial_information
							where debtor_id = debt.debtor_id and
							debtor_number = 1 and 
							vehicle_year_make_model = VACSWorker) 
					End
				Else 
					Case When IsNull(worker_id, '') = '' Then @county_code Else worker_id 
					End
			End,		
	"Status Code" = (select calsaws_code		
				from CalSAWSRecoveryAccountStatusMap
				where CalSAWSRecoveryAccountStatusMap.vacs_code = debt.debt_status),

	"Status Reason Code" = Case When (debtor1_intercept_submitted= 'Y' or debtor2_intercept_submitted = 'Y') and
					(select calsaws_code		
					from CalSAWSRecoveryAccountStatusReasonMap
					where CalSAWSRecoveryAccountStatusReasonMap.vacs_code = debt.debt_status) in ('AC', 'SU') Then  'TI' 
				Else (select calsaws_code		
					from CalSAWSRecoveryAccountStatusReasonMap
					where CalSAWSRecoveryAccountStatusReasonMap.vacs_code = debt.debt_status) End,

	"Reason Code" = Case When IsNull((select calsaws_code	
		from CalSAWSRecoveryAccountReasonMap
		where CalSAWSRecoveryAccountReasonMap.vacs_code = debt.debt_reason_code), '') = '' Then 'OH' 
		Else (select calsaws_code	
		from CalSAWSRecoveryAccountReasonMap
		where CalSAWSRecoveryAccountReasonMap.vacs_code = debt.debt_reason_code) End,

	"Status Date" = (select max(status_date)
			from debt_status_history
			where debt_number = debt.debt_number),

	"Investigation Code" = 'AN',

	"Current Balance" = Case When balance_amount < 0 Then 0 Else balance_amount End,

	"Cause Code" = Case When type_code = 'FOO' and
				debt_type_code = 'ADM' and
				date_entered < '20000301' 
				Then 'AP'
				Else 
					(select calsaws_code		
					from CalSAWSRecoveryAccountCauseMap
					where CalSAWSRecoveryAccountCauseMap.debt_type_code = debt.debt_type_code and
						CalSAWSRecoveryAccountCauseMap.type_code = debt.type_code) 
				End,

	"Cause Date" = date_entered,

	"Final demand Notice Date" = '12/01/9999',

	"Non-CalWIN Case Indicator" = Case When Not Exists (select claim_number from cw_claim_number where cw_claim_number.debt_number = debt.debt_number)
					Then 'Y' Else 'N' End					
		

from debt
JOIN debtor ON debtor.debtor_id = debt.debtor_id
JOIN case_info ON case_info.reference_number = debt.reference_number
JOIN DebtList ON DebtList.debt_number = debt.debt_number
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number
         
go

grant execute on FillRecoveryAccount to public
go

/*   	
	procedure:	FillRecoveryAccountResponsibleParty.prc
	written by:    	Craig Deane
     	date written:  	September 2020
     	database:      	welcoll
     	function:      	This fills the RecoveryAccountResponsibleParty table with debtor data
     	modified by:  	Craig Deane 12/3/2020 FSD 1.4
     	changes:	Added support for PhysicalAddressEffectiveDate, MailingAddressEffectiveDate, Language, WrittenLanguage, and PersonNumberIdentifier
     	modified by:   	Craig Deane 2/2021
     	changes:	Modified to use the DebtList table to filter results
     	modified by:   	Craig Deane 3/18/2021
     	changes:       	Modified to use the new LegacyAccountNumberIdentifierFunction table valued function	
     	modified by:   	Craig Deane 4/28/2021 FSD 1.7
     	changes:	Modified the processing for Organization Name and Debtor Type. Fixed the Language processing
     	modified by:   	Craig Deane 6/10/2021 FSD 1.7
     	changes:	Made it use the Private Mail Box or Care of Address when the Street Address is blank		
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountResponsibleParty'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountResponsibleParty
  END
go

create procedure dbo.FillRecoveryAccountResponsibleParty
as

declare 	@serial_type_ind	char(1),
	  	@program_status_ind char(1)

select @serial_type_ind = serial_type
  from calwin_control
 where calwin_key = 'C'

select @program_status_ind = IsNull(program_status_ind, 'N')
  from tax_intercept_control
  where tax_intercept_key = 'I'


delete RecoveryAccountResponsibleParty

insert RecoveryAccountResponsibleParty
select 	"Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,

	"Case Number" = (select case when @serial_type_ind = 'V' then substring(debt.reference_number, 5, 7) else
					substring(debt.reference_number, 3, 7) 	end),

	"Person Identifier" = cin1,

	"Organization ID" = provider_number,

	"Adjustment Indicator" = Case When @program_status_ind = 'Y' and IsNull(debt.program_status, case_status) = 'A' or
						@program_status_ind = 'N' and IsNull(case_status, debt.program_status) = 'A' Then 'Y' 
					Else 'N' End,

	"Bill Indicator" = Case When statement_ind = 'Y' and debtor1_statement_ind = 'Y' Then 'Y' Else 'N' End,

	"Tax Intercept Indicator" = IsNull(debtor1_ftb_intercept_ind, 'N'),

	"Responsible Party Status Date" = date_entered,

	"Delinquent Date" = delinquency_date, 

	"Debtor Type" = Case When IsNull(provider_number, '') <> '' or IsNull(debtor1_name_mid_init, '') = '*' Then 'C' Else 'I' End,

	"Responsible Party First Name" = Case When IsNull(provider_number, '') = '' and IsNull(debtor1_name_mid_init, '') <> '*' Then debtor1_name_first End,

	"Responsible Party Middle Name" = Case When IsNull(provider_number, '') = '' and IsNull(debtor1_name_mid_init, '') <> '*' Then debtor1_name_mid_init End,

	"Responsible Party Last Name" = Case When IsNull(provider_number, '') = '' and IsNull(debtor1_name_mid_init, '') <> '*' Then 
					(select last_name from dbo.NameSuffixFunction(debtor1_name_last)) End, 

	"Responsible Party Suffix" = Case When IsNull(provider_number, '') = '' and IsNull(debtor1_name_mid_init, '') <> '*' Then 
					(select suffix_name from dbo.NameSuffixFunction(debtor1_name_last)) End, 

	"Responsible Party Status" = 'AC',

	"Organization Name" = Case When IsNull(provider_number, '') <> '' or IsNull(debtor1_name_mid_init, '') = '*' Then debtor1_name_last End,

	"AKA First Name" = first_name,

	"AKA Middle Name" = '',

	"AKA Last Name" = (select last_name from dbo.NameSuffixFunction(last_name)),

	"AKA Suffix" = (select suffix_name from dbo.NameSuffixFunction(last_name)),

	"Coresponsible Flag" = Case When IsNull(debtor2_name_last, '') != '' Then 'Y' Else 'N' End,

	"Physical Address - Effective Date" = null,

	"Physical Address - Line 1 Address" = '',

	"Physical  Address - Line 2 Address" = '',

	"Physical  Address - City Name" = '',

	"Physical  Address- State Code" = '',

	"Physical  Address - Zip Code" = '',

	"Mailing Address - Effective Date" = dbo.AddressEffectiveDateFunction(debtor.debtor_id),

	"Mailing Address- Line 1 Address" = Case When IsNull(debtor1_street_address, '') <> '' Then (select address_line1 from AddressFunction(debtor1_street_address)) Else
						Case When IsNull(debtor1_private_mailbox, '') <> '' Then debtor1_private_mailbox Else
							Case When IsNull(debtor1_care_of_address, '') <> '' Then debtor1_care_of_address End 
						End 
					    End,

	"Mailing Address- Line 2 Address" = Case When IsNull(debtor1_street_address, '') <> '' Then (select address_line2 from AddressFunction(debtor1_street_address)) End,

	"Mailing Address - City Name" = debtor1_city,

	"Mailing Address- State Code" = debtor1_state,

	"Mailing Address - Zip Code" =  debtor1_zip,

	"Phone Number - Area Code" = substring(debtor1_phone, 1, 3),

	"Phone Number" = substring(debtor1_phone, 4, 7),

	"Phone Extension Number" = '',

	"Responsible Party Date of Birth" = debtor1_dob,

	"Responsible Party SSN" = debtor1_ssn,

	"Responsible Party Driver License Number" = debtor1_drivers_license,

	"Language" = Case When IsNull((select calsaws_code		
			from CalSAWSLanguageCodeMap
			where CalSAWSLanguageCodeMap.vacs_code = debtor.primary_language_code), '') = '' Then '03'
			Else (select calsaws_code		
			from CalSAWSLanguageCodeMap
			where CalSAWSLanguageCodeMap.vacs_code = debtor.primary_language_code) End,

	"Written Language" = '',

	"Person Number Identifier" = '01'

from debt
INNER JOIN debtor ON debt.debtor_id = debtor.debtor_id
INNER JOIN DebtList ON debt.debt_number = DebtList.debt_number
INNER JOIN case_info ON case_info.reference_number = debt.reference_number
LEFT OUTER JOIN alias_names ON debt.debtor_id = alias_names.debtor_id
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number

where alias_sequence_number = (select max(alias_sequence_number)
				from alias_names
				where debtor_id = debt.debtor_id) or 
				alias_sequence_number is null

union

select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,

	"Case Number" = (select case when @serial_type_ind = 'V' then substring(debt.serial_number, 3, 7) else
					substring(debt.serial_number, 1, 7) 	end),

	"Person Identifier" = cin2,

	"Organization ID" = '',

	"Adjustment Indicator" = Case When @program_status_ind = 'Y' and IsNull(debt.program_status, case_status) = 'A' or
						@program_status_ind = 'N' and IsNull(case_status, debt.program_status) = 'A' Then 'Y' 
					Else 'N' End,	

	"Bill Indicator" = Case When statement_ind = 'Y' and debtor2_statement_ind = 'Y' Then 'Y' Else 'N' End,

	"Tax Intercept Indicator" = IsNull(debtor2_ftb_intercept_ind, 'N'),

	"Responsible Party Status Date" = date_entered,

	"Delinquent Date" = delinquency_date, 

	"Debtor Type" = 'I',

	"Responsible Party First Name" = debtor2_name_first,

	"Responsible Party Middle Name" = debtor2_name_mid_init,

	"Responsible Party Last Name" = (select last_name from dbo.NameSuffixFunction(debtor2_name_last)),

	"Responsible Party Suffix" = (select suffix_name from dbo.NameSuffixFunction(debtor2_name_last)),

	"Responsible Party Status" = 'AC',

	"Organization Name" = '',

	"AKA First Name" = first_name,

	"AKA Middle Name" = '',

	"AKA Last Name" = (select last_name from dbo.NameSuffixFunction(last_name)),

	"AKA Suffix" = (select suffix_name from dbo.NameSuffixFunction(last_name)),

	"Coresponsible Flag" = 'Y',

	"Physical Address - Effective Date" = null,

	"Physical Address- Line 1 Address" = '',

	"Physical Address- Line 2 Address" = '',

	"Physical Address - City Name" = '',

	"Physical Address- State Code" = '',

	"Physical Address - Zip Code" = '',

	"Mailing Address - Effective Date" = dbo.AddressEffectiveDateFunction(dbo.debtor.debtor_id),

	"Mailing Address- Line 1 Address" = Case When IsNull(debtor2_street_address, '') + IsNull(debtor2_care_of_address, '') + IsNull(debtor2_private_mailbox, '') = '' Then 
					         Case When IsNull(debtor1_street_address, '') <> '' Then (select address_line1 from AddressFunction(debtor1_street_address)) Else
						     Case When IsNull(debtor1_private_mailbox, '') <> '' Then debtor1_private_mailbox Else
						         Case When IsNull(debtor1_care_of_address, '') <> '' Then debtor1_care_of_address End 
						     End 
					         End
						 Else
						 Case When IsNull(debtor2_street_address, '') <> '' Then (select address_line1 from AddressFunction(debtor2_street_address)) Else
						     Case When IsNull(debtor2_private_mailbox, '') <> '' Then debtor2_private_mailbox Else
						     	Case When IsNull(debtor2_care_of_address, '') <> '' Then debtor2_care_of_address End 
						     End 
					         End
					     End,

	"Mailing Address- Line 2 Address" = Case When IsNull(debtor2_street_address, '') + IsNull(debtor2_care_of_address, '') + IsNull(debtor2_private_mailbox, '') = '' Then
						(select address_line2 from AddressFunction(debtor2_street_address))
						Else (select address_line2 from AddressFunction(debtor1_street_address))
					     End,

	"Mailing Address - City Name" = Case When IsNull(debtor2_street_address, '') + IsNull(debtor2_care_of_address, '') + IsNull(debtor2_private_mailbox, '') = '' Then debtor1_city Else debtor2_city End,

	"Mailing Address- State Code" = Case When IsNull(debtor2_street_address, '') + IsNull(debtor2_care_of_address, '') + IsNull(debtor2_private_mailbox, '') = '' Then debtor1_state Else debtor2_state End,

	"Mailing Address - Zip Code" =  Case When IsNull(debtor2_street_address, '') + IsNull(debtor2_care_of_address, '') + IsNull(debtor2_private_mailbox, '') = '' Then debtor1_zip Else debtor2_zip End,

	"Phone Number - Area Code" = substring(debtor2_phone, 1, 3),

	"Phone Number" = substring(debtor2_phone, 4, 7),

	"Phone Extension Number" = '',

	"Responsible Party Date of Birth" = debtor2_dob,

	"Responsible Party SSN" = debtor2_ssn,

	"Responsible Party Driver License Number" = debtor2_drivers_license,

	"Language" = Case When IsNull((select calsaws_code		
			from CalSAWSLanguageCodeMap
			where CalSAWSLanguageCodeMap.vacs_code = debtor.primary_language_code), '') = '' Then '03'
			Else (select calsaws_code		
			from CalSAWSLanguageCodeMap
			where CalSAWSLanguageCodeMap.vacs_code = debtor.primary_language_code) End,

	"Written Language" = '',

	"Person Number Identifier" = '02'
					
from debt
INNER JOIN debtor ON debt.debtor_id = debtor.debtor_id
INNER JOIN DebtList ON debt.debt_number = DebtList.debt_number
INNER JOIN case_info ON case_info.reference_number = debt.reference_number
LEFT OUTER JOIN alias_names ON debt.debtor_id = alias_names.debtor_id
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number
where IsNull(debtor2_name_last, '') != '' and
(alias_sequence_number = (select max(alias_sequence_number)
				from alias_names
				where debtor_id = debt.debtor_id) or 
				alias_sequence_number is null)
 

update RecoveryAccountResponsibleParty
set PhysicalAddressEffectiveDate = MailingAddressEffectiveDate,
PhysicalAddressLine1Address = MailingAddressLine1Address,
PhysicalAddressLine2Address = MailingAddressLine2Address,
PhysicalAddressCityName = MailingAddressCityName,
PhysicalAddressStateCode = MailingAddressStateCode,
PhysicalAddressZipCode = MailingAddressZipCode,
WrittenLanguage = Language
            
go

grant execute on FillRecoveryAccountResponsibleParty to public
go

/*   procedure:     FillRecoveryAccountRepaymentTerm.prc
     written by:    Craig Deane
     date written:  September 2020
     database:      welcoll
     function:      This fills the RecoveryAccountRepaymentTerm table with claim data
     modified by:   Craig Deane 2/2021
     changes: 	    Modified to use the DebtList table to filter results
     modified by:   Craig Deane 3/17//2021
     changes:       Modified to use the new LegacyAccountNumberIdentifierFunction table valued function	
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountRepaymentTerm'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountRepaymentTerm
  END
go

create procedure dbo.FillRecoveryAccountRepaymentTerm
as

declare 	@serial_type_ind	char(1)


select @serial_type_ind = serial_type
  from calwin_control
 where calwin_key = 'C'

delete RecoveryAccountRepaymentTerm

insert RecoveryAccountRepaymentTerm
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,
	"Case Number" = (select case when @serial_type_ind = 'V' then substring(debt.reference_number, 5, 7) else
					substring(debt.reference_number, 3, 7) 	end),
	"Person Identifier" = cin1,
	"Organization ID" = provider_number,
	"Override Cause Indicator" = 'Y',
	"Repay Percent" = '000',
	"Repay Amount" = installment_amount,
	"Frequency Code" = 'MO',
	"Begin Date" = date_first_payment_due,
	"Type Code" = 'EP',
	"End Date" = date_last_payment_due,
	"Agreement Type" = '',
	"Entitlement Type" = ''
					
from debt
JOIN debtor ON debtor.debtor_id = debt.debtor_id
JOIN case_info ON case_info.reference_number = debt.reference_number
JOIN payment_plan ON payment_plan.plan_id = debt.plan_id
JOIN DebtList ON DebtList.debt_number = debt.debt_number
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number
             
go

grant execute on FillRecoveryAccountRepaymentTerm to public
go

/*   
	procedure:	FillRecoveryAccountOverpayment.prc
     	written by:    	Craig Deane
     	date written:  	September 2020
     	database:      	welcoll
     	function:      	This fills the Recovery Account Overpayment table with claim data
     	modified by:   	Craig Deane 2/2021
     	changes: 	Modified to use the DebtList table to filter results	
     	modified by:   	Craig Deane 3/18//2021
     	changes:       	Modified to use the new LegacyAccountNumberIdentifierFunction table valued function
     	modified by:   	Craig Deane 4/28/2021 FSD 1.7
     	changes:	Fixed the Aid Code processing
     	modified by:   	Craig Deane 6/10/2021 FSD 1.7 
     	changes:	Added the option for the Effective Date
     	modified by:   	Craig Deane 7/28/2021 FSD 1.8
     	changes:	Fixed a bug where additional charges were being added to the Amounts
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountOverpayment'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountOverpayment
  END
go

create procedure dbo.FillRecoveryAccountOverpayment
as

declare @transaction_date_ind	integer

select @transaction_date_ind = transaction_date_ind
from CalSAWSSettings
where key_value = 1

delete RecoveryAccountOverpayment

insert RecoveryAccountOverpayment
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,

	"Overpayment Amount" = financial_detail.amount,

	"Effective Date" = Case When @transaction_date_ind = 0 
				Then transaction_control.date_created
				Else transaction_control.date_effective
			   End,	
									
	"Original Amount" = financial_detail.amount,		
							
	"Corrected Amount" = 0,		
						
	"Aid Code" = Case When IsNull((select calsaws_code		
		from CalSAWSAidCodeMap
		where CalSAWSAidCodeMap.vacs_code = SUBSTRING(debt.reference_number, 1, 2)), '') = '' Then SUBSTRING(debt.reference_number, 1, 2)
		Else (select calsaws_code		
		from CalSAWSAidCodeMap
		where CalSAWSAidCodeMap.vacs_code = SUBSTRING(debt.reference_number, 1, 2)) End

from debt
INNER JOIN financial_detail ON debt.debt_number = financial_detail.debt_number
INNER JOIN DebtList ON debt.debt_number = DebtList.debt_number
INNER JOIN transaction_control ON financial_detail.transaction_number = transaction_control.transaction_number
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number

where 	transaction_code = 'OVP' and
	charge_code = 'DEB'

go

grant execute on FillRecoveryAccountOverpayment to public
go

/*   
	procedure:     	FillRecoveryAccountTransaction.prc
     	written by:    	Craig Deane
     	date written:  	10/12/2020
     	database:      	welcoll
     	function:      	This fills the RecoveryAccountTransaction table with transaction data
     	modified by:   	Craig Deane 2/2021
     	changes:	Modified to use the DebtList table to filter results
     	modified by:   	Craig Deane 3/3/2021
     	changes:	Set the Backed Out Indicator to blank	
     	modified by:   	Craig Deane 3/18//2021
     	changes:       	Modified to use the new LegacyAccountNumberIdentifierFunction table valued function
     	modified by:   	Craig Deane 6/10/2021 FSD 1.7 
     	changes:	Added the option for the Effective Date
     	modified by:   	Craig Deane 7/28/2021 FSD 1.8
     	changes:	Changed the sign on the amount to conform to CalSAWS
 */

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountTransaction'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountTransaction
  END
go

create procedure dbo.FillRecoveryAccountTransaction
as

declare @transaction_date_ind	integer

select @transaction_date_ind = transaction_date_ind
from CalSAWSSettings
where key_value = 1

delete RecoveryAccountTransaction 

insert RecoveryAccountTransaction 
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,

 	"Transaction Number Identifier" = financial_detail.transaction_number,
	"Transaction Amount" = - financial_detail.amount,
	"Transaction Type Code" = (select calsaws_code		
				from CalSAWSTransactionTypeMap
				where CalSAWSTransactionTypeMap.vacs_code = transaction_control.transaction_code),
	"Transaction Delivery Type Code" = 'SY',
	"Effective Date" = Case When @transaction_date_ind = 0 
				Then transaction_control.date_created
				Else transaction_control.date_effective
			   End,	
	"Posted Date" = transaction_control.date_created,
	"Backed Out Indicator" = '',
	"Posted Status Code" = 'PR'
					
					
from debt
JOIN financial_detail ON financial_detail.debt_number = debt.debt_number
JOIN transaction_control ON transaction_control.transaction_number = financial_detail.transaction_number
JOIN DebtList ON DebtList.debt_number = debt.debt_number
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number

where transaction_control.transaction_code <> 'OVP' and
	financial_detail.charge_code = 'DEB'
             
go

grant execute on FillRecoveryAccountTransaction to public
go

/*   procedure:     FillRecoveryAccountTransactionDetail.prc
     written by:    Craig Deane
     date written:  10/13/2020
     database:      welcoll
     function:      This fills the RecoveryAccountTransactionDetail table with transaction data
     modified by:   Craig Deane 2/2021
     changes: 	    Modified to use the DebtList table to filter results	
     modified by:   Craig Deane 3/3/2021
     changes:	    Modified to use the CleanMemoFunction function
     modified by:   Craig Deane 3/18//2021
     changes:       Modified to use the new LegacyAccountNumberIdentifierFunction table valued function
     modified by:   Craig Deane 4/28/2021 FSD 1.7
     changes:	    Fixed the Transaction Description processing
 */

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountTransactionDetail'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountTransactionDetail
  END
go

create procedure dbo.FillRecoveryAccountTransactionDetail
as

delete RecoveryAccountTransactionDetail 

insert RecoveryAccountTransactionDetail 
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,
 	"Transaction Number Identifier" = financial_detail.transaction_number,
	"Transaction Description" = substring(dbo.CleanMemoFunction(activity_code + 
				IsNull((select ' - ' + description from activity where activity_code = user_activity.activity_code), '') +  						IsNull((select ' - ' + logon_id from users where user_number = user_activity.user_number), '') + 
				IsNull( ' - ' + memo, '- No Memo Entered') + 
				IsNull(convert(varchar(max), memo_text), '')), 1, 2000)
					
from financial_detail
JOIN transaction_control ON financial_detail.transaction_number = transaction_control.transaction_number
JOIN debt_user_activity ON financial_detail.debt_number = debt_user_activity.debt_number
JOIN user_activity ON debt_user_activity.activity_number = user_activity.activity_number
JOIN DebtList ON DebtList.debt_number = financial_detail.debt_number
JOIN LegacyAccountNumberIdentifierFunction() ON financial_detail.debt_number = LegacyAccountNumberIdentifierFunction.debt_number

where  	transaction_control.date_created = user_activity.date_of_activity and
	transaction_control.transaction_code <> 'OVP' and
	financial_detail.charge_code = 'DEB'
             
go

grant execute on FillRecoveryAccountTransactionDetail to public
go

/*   procedure:     FillRecoveryAccountCaseNotes.prc
     written by:    Craig Deane
     date written:  10/13/2020
     database:      welcoll
     function:      This fills the FillRecoveryAccountCaseNotes table with Activity memo data
     modified by:   Craig Deane 2/2021
     changes: 	    Modified to use the DebtList table to filter results
     modified by:   Craig Deane 3/3/2021
     changes:	    Modified to use the CleanMemoFunction function	
     modified by:   Craig Deane 3/18//2021
     changes:       Modified to use the new LegacyAccountNumberIdentifierFunction table valued function
     modified by:   Craig Deane 4/28/2021 FSD 1.7
     changes:	    Fixed the Note Description processing. Added unmapped Aid Code support
 */

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'FillRecoveryAccountCaseNotes'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.FillRecoveryAccountCaseNotes
  END
go

create procedure dbo.FillRecoveryAccountCaseNotes
as

delete RecoveryAccountCaseNotes

insert RecoveryAccountCaseNotes
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,
 	"Note Description" = substring(dbo.CleanMemoFunction(activity_code + 
				IsNull((select ' - ' + description from activity where activity_code = user_activity.activity_code), '') +  						IsNull((select ' - ' + logon_id from users where user_number = user_activity.user_number), '') + 
				IsNull( ' - ' + memo, '- No Memo Entered') + 
				IsNull(convert(varchar(max), memo_text), '')), 1, 2000),
	"Date Note Entered" = date_of_activity
			
from debt

JOIN debt_user_activity ON debt.debt_number = debt_user_activity.debt_number
JOIN user_activity ON debt_user_activity.activity_number = user_activity.activity_number
JOIN DebtList ON DebtList.debt_number = debt.debt_number
JOIN LegacyAccountNumberIdentifierFunction() ON debt.debt_number = LegacyAccountNumberIdentifierFunction.debt_number
 
where memo is not null
             

insert RecoveryAccountCaseNotes
select "Legacy Account Number Identifier" = LegacyAccountNumberIdentifier,
 	"Note Description" = 'Aid Code: ' + AidCode + IsNull(' - ' + calsaws_description, ''),
	"Date Note Entered" = getdate()
					
					
from RecoveryAccountOverpayment

JOIN CalSAWSAidCodeMap ON RecoveryAccountOverpayment.AidCode = CalSAWSAidCodeMap.vacs_code
Where IsNull(CalSAWSAidCodeMap.calsaws_code, '') = ''

go

grant execute on FillRecoveryAccountCaseNotes to public
go

/*	
	procedure:	CreateDebtList.prc
	written by:    	Craig Deane
	date written:  	2/3/2021
	database:      	welcoll
	function:     	Generates the list of debts to be extracted
	modified by:   	Craig Deane 2/9/2021
	changes:	Modified to use the CalSAWSSettings table	
	modified by:   	Craig Deane 5/24/2021
	changes:	Added filters for exclude_non_bic_ind, exclude_indicator, and exclude_zero_balance_indicator. Fixed a bug in the percentage proceesing 	
	modified by:   	Craig Deane 6/10/2021	FSD 1.7
	changes:	Added options for the date cutoff: Date Debt Status, Date of Discovery, and Date Entered
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'CreateDebtList'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.CreateDebtList
  END
go

create procedure dbo.CreateDebtList
as

declare @percentage 		dec(5, 2),
	@years			integer,
	@months			integer,
	@days			integer,
	@date_cutoff		datetime,
	@date_cutoff_ind	integer,
	@other_reason_text_ind	char(1),
	@exclude_non_bic_ind	char(1),
	@cutoff_date_field_ind	integer,
	@cutoff_date_field	datetime

select @percentage = data_set_percentage,
	@years = date_cutoff_years,
	@months = date_cutoff_months,
	@days = date_cutoff_days,
	@other_reason_text_ind = other_reason_text_ind,	
	@exclude_non_bic_ind = exclude_non_bic_ind,
	@cutoff_date_field_ind = cutoff_date_field_ind
from CalSAWSSettings 
where key_value = 1

if IsNull(@percentage, 100) < 1 
	select @percentage = 1

if IsNull(@percentage, 100) > 100
	select @percentage = 100

select @percentage = 100 / @percentage

select @date_cutoff_ind = IsNull(@years, 0) + IsNull(@months, 0) + IsNull(@days, 0)

if @date_cutoff_ind > 0
begin
  select @date_cutoff = DATEADD(Year, - IsNull(@years, 0), GETDATE()) 
  select @date_cutoff = DATEADD(Month, - IsNull(@months, 0), @date_cutoff) 
  select @date_cutoff = DATEADD(Day, - IsNull(@days, 0), @date_cutoff) 
end

delete DebtList

insert DebtList
  select debt_number
  from debt
  where floor(debt_number % @percentage) = 0

delete DebtList
from debt
where DebtList.debt_number = debt.debt_number and
	balance_amount = 0 and 
	(date_debt_status < @date_cutoff and @cutoff_date_field_ind = 0 or
	 date_of_discovery < @date_cutoff and @cutoff_date_field_ind = 1 or
	 date_entered < @date_cutoff and @cutoff_date_field_ind = 2) and
	(IsNull(other_reason_text, '') = '' or @other_reason_text_ind = 'N') and
	@date_cutoff_ind > 0

delete DebtList
from debt
where DebtList.debt_number = debt.debt_number and
	balance_amount < 0 

delete DebtList
from debt
where DebtList.debt_number = debt.debt_number and
		type_code in (select vacs_code
			from CalSAWSProgramCodeMap
			where exclude_indicator = 1)

delete DebtList
from debt
where DebtList.debt_number = debt.debt_number and
		debt_status in (select vacs_code
			from CalSAWSRecoveryAccountStatusMap
			where exclude_zero_balance_indicator = 1)

delete DebtList
from debt
where DebtList.debt_number = debt.debt_number and
	IsNull(bic_code, '') = '' and
	@exclude_non_bic_ind = 'Y'

go

grant execute on CreateDebtList to public
go

/*   procedure:     PriorityNumbers.prc
     written by:    Craig Deane
     date written:  10/7/2020
     database:      welcoll
     function:      Generates the Priority Numbers for the Recovery Account
     modified by:  
     changes: 	 
*/

use welcoll
go

IF EXISTS(SELECT 1
          FROM   INFORMATION_SCHEMA.ROUTINES
          WHERE  ROUTINE_NAME = 'PriorityNumbers'
                 AND SPECIFIC_SCHEMA = 'dbo')
  BEGIN
      DROP PROCEDURE dbo.PriorityNumbers
  END
go

create procedure dbo.PriorityNumbers
as

declare @priority_number	integer,
	@serial_number		char(9),
	@debt_number		integer,
	@temp_serial_number	char(9)

delete PriorityNumber

DECLARE priority_numbers CURSOR
    FOR select serial_number,
		debt.debt_number
	from debt, DebtList
	where debt.debt_number = DebtList.debt_number
	order by serial_number, debt_type_code desc, date_entered

OPEN priority_numbers

select @priority_number = 0

FETCH NEXT FROM priority_numbers INTO @serial_number, @debt_number

select @temp_serial_number = @serial_number

WHILE @@FETCH_STATUS = 0  
    BEGIN
	if @serial_number = @temp_serial_number
	begin
	   select @priority_number = @priority_number + 1
	end
	else
	begin
	    select @priority_number = 1
	    select @temp_serial_number = @serial_number
	end

	insert PriorityNumber
	select @priority_number, @serial_number, @debt_number

        FETCH NEXT FROM priority_numbers INTO @serial_number, @debt_number

	

    END


CLOSE priority_numbers

DEALLOCATE priority_numbers

go

grant execute on PriorityNumbers to public
go

